@@ -3,12 +3,12 @@ use warnings;
 
 package Pad::Tie;
 
-use Lexical::Persistence;
+use Pad::Tie::LP;
 use Data::OptList;
 use Module::Pluggable require => 1;
 use Carp ();
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 my %METHOD;
 
 sub new {
@@ -18,7 +18,8 @@ sub new {
   my $self = bless {
     invocant => $invocant,
     methods  => $methods,
-    persist  => Lexical::Persistence->new,
+    persist  => Pad::Tie::LP->new,
+    pre_call => [],
   } => $class;
   $self->build_context;
   #tie %{ $self->{context} }, 'Pad::Tie::Context', $self;
@@ -38,16 +39,24 @@ sub build_methods {
 
 sub build_context {
   my $self = shift;
+  my $methods = shift || $self->{methods};
   $self->{context} ||= {};
   $self->build_methods unless %METHOD;
-  for (@{ $self->{methods} }) {
+  for (@$methods) {
     my ($method_personality, $plugin_arg) = @$_;
     Carp::confess "unhandled method personality: $method_personality"
       unless $METHOD{$method_personality};
-    $METHOD{$method_personality}->$method_personality(
+    my $plugin = $METHOD{$method_personality};
+    my $rv = $plugin->$method_personality(
       $self->{context},
       $self->{invocant}, $plugin_arg,
-    ),
+    ) || {};
+    # XXX I hate this but I can't think of a better way to do it offhand.
+    # if you aren't me, don't use this; talk to me about it instead. -- hdp,
+    # 2007-04-24
+    if ($rv->{pre_call}) {
+      push @{ $self->{pre_call} }, @{ $rv->{pre_call} };
+    }
   }
 }
 
@@ -64,6 +73,7 @@ sub clone {
 
 sub call {
   my ($self, $code, @args) = @_;
+  $_->($self, $code, \@args) for @{ $self->{pre_call} };
   return $self->{persist}->call($code, @args);
 }
 
@@ -82,7 +92,7 @@ Pad::Tie - tie an object to lexical contexts
 
 =head1 VERSION
 
- Version 0.002
+ Version 0.003
 
 =head1 SYNOPSIS
 
@@ -227,32 +237,6 @@ need to pass an explicit C<undef> value.  See L</SYNOPSIS>.
 More detail about methods and personalities is given above.  See
 L<DESCRIPTION|/DESCRIPTION>.
 
-=cut 
-
-=begin later
-
-Some extra options may be passed in as a third hashref argument to C<new>:
-
-=over 
-
-=item * context_name
-
-Sets the context name used by Lexical::Persistence.  By default, the bound object's
-methods are available in the C<_> context (C<$foo>).  Pass in this option to
-give them a different prefix, e.g. C<$self_foo>.  See
-L<Lexical::Persistence/set_context>.
-
-=item * cache
-
-Determines whether or not to cache the results of method calls within a single
-method call.  The default is true; pass in a false value to disable result
-caching.  Note that this only affects a limited number of method personalities;
-see the individual plugins' documentation for details.
-
-=back
-
-=end
-
 =head1 METHODS
 
 Most of the time you will only need to use C<new> and C<call>, or perhaps
@@ -287,6 +271,10 @@ e.g. L<Rose::Object::MakeMethods::Generic|Rose::Object::MakeMethods::Generic>'s
 different kinds of hash/array accessors, a scalar that calls different methods
 for FETCH and STORE (C<$url> in examples/mech.pl).
 
+=item * more options
+
+interface for configuring the underlying Lexical::Persistence object
+
 =head1 SEE ALSO
 
 L<Pad::Tie::Plugin>
